@@ -1,24 +1,24 @@
 /* engine-gpgsm.c - GpgSM engine.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009,
-                 2010 g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009,
+ *               2010 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -37,7 +37,6 @@
 #include <locale.h>
 #endif
 #include <fcntl.h> /* FIXME */
-#include <errno.h>
 
 #include "gpgme.h"
 #include "util.h"
@@ -83,6 +82,7 @@ struct engine_gpgsm
   iocb_data_t output_cb;
 
   iocb_data_t message_cb;
+  iocb_data_t diag_cb;
 
   struct
   {
@@ -107,7 +107,12 @@ struct engine_gpgsm
 
   gpgme_data_t inline_data;  /* Used to collect D lines.  */
 
+  char request_origin[10];
+
   struct gpgme_io_cbs io_cbs;
+
+  /* Memory data containing diagnostics (--logger-fd) of gpgsm */
+  gpgme_data_t diagnostics;
 };
 
 typedef struct engine_gpgsm *engine_gpgsm_t;
@@ -145,6 +150,13 @@ close_notify_handler (int fd, void *opaque)
 	(*gpgsm->io_cbs.remove) (gpgsm->status_cb.tag);
       gpgsm->status_cb.fd = -1;
       gpgsm->status_cb.tag = NULL;
+      /* Because the server keeps on running as long as the
+       * gpgme_ctx_t is valid the diag fd will not receive a close and
+       * thus the operation gets stuck trying to read the diag fd.
+       * The status fd however is closed right after it received the
+       * "OK" from the command.  So we use this event to also close
+       * the diag fd.  */
+      _gpgme_io_close (gpgsm->diag_cb.fd);
     }
   else if (gpgsm->input_cb.fd == fd)
     {
@@ -177,6 +189,13 @@ close_notify_handler (int fd, void *opaque)
       gpgsm->message_cb.fd = -1;
       gpgsm->message_cb.tag = NULL;
     }
+  else if (gpgsm->diag_cb.fd == fd)
+    {
+      if (gpgsm->diag_cb.tag)
+	(*gpgsm->io_cbs.remove) (gpgsm->diag_cb.tag);
+      gpgsm->diag_cb.fd = -1;
+      gpgsm->diag_cb.tag = NULL;
+    }
 }
 
 
@@ -212,6 +231,8 @@ gpgsm_cancel (void *engine)
     _gpgme_io_close (gpgsm->output_cb.fd);
   if (gpgsm->message_cb.fd != -1)
     _gpgme_io_close (gpgsm->message_cb.fd);
+  if (gpgsm->diag_cb.fd != -1)
+    _gpgme_io_close (gpgsm->diag_cb.fd);
 
   if (gpgsm->assuan_ctx)
     {
@@ -233,6 +254,8 @@ gpgsm_release (void *engine)
 
   gpgsm_cancel (engine);
 
+  gpgme_data_release (gpgsm->diagnostics);
+
   free (gpgsm->colon.attic.line);
   free (gpgsm);
 }
@@ -245,17 +268,18 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
   gpgme_error_t err = 0;
   engine_gpgsm_t gpgsm;
   const char *pgmname;
-  const char *argv[5];
+  const char *argv[7];
+  char *diag_fd_str = NULL;
   int argc;
-#if !USE_DESCRIPTOR_PASSING
   int fds[2];
-  int child_fds[4];
-#endif
+  int child_fds[5];
+  int nchild_fds;
   char *dft_display = NULL;
   char dft_ttyname[64];
   char *env_tty = NULL;
   char *dft_ttytype = NULL;
   char *optstr;
+  unsigned int connect_flags;
 
   (void)version; /* Not yet used.  */
 
@@ -283,6 +307,11 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
   gpgsm->message_cb.tag = 0;
   gpgsm->message_cb.server_fd = -1;
   *gpgsm->message_cb.server_fd_str = 0;
+  gpgsm->diag_cb.fd = -1;
+  gpgsm->diag_cb.dir = 1;
+  gpgsm->diag_cb.tag = 0;
+  gpgsm->diag_cb.server_fd = -1;
+  *gpgsm->diag_cb.server_fd_str = 0;
 
   gpgsm->status.fnc = 0;
   gpgsm->colon.fnc = 0;
@@ -299,7 +328,20 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
   gpgsm->io_cbs.event = NULL;
   gpgsm->io_cbs.event_priv = NULL;
 
-#if !USE_DESCRIPTOR_PASSING
+  if (_gpgme_io_pipe (fds, 1) < 0)
+    {
+      err = gpg_error_from_syserror ();
+      goto leave;
+    }
+  gpgsm->diag_cb.fd = fds[0];
+  gpgsm->diag_cb.server_fd = fds[1];
+
+#if USE_DESCRIPTOR_PASSING
+  child_fds[0] = gpgsm->diag_cb.server_fd;
+  child_fds[1] = -1;
+  nchild_fds = 2;
+  connect_flags = ASSUAN_PIPE_CONNECT_FDPASSING;
+#else /*!USE_DESCRIPTOR_PASSING*/
   if (_gpgme_io_pipe (fds, 0) < 0)
     {
       err = gpg_error_from_syserror ();
@@ -327,8 +369,11 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
   child_fds[0] = gpgsm->input_cb.server_fd;
   child_fds[1] = gpgsm->output_cb.server_fd;
   child_fds[2] = gpgsm->message_cb.server_fd;
-  child_fds[3] = -1;
-#endif
+  child_fds[3] = gpgsm->diag_cb.server_fd;
+  child_fds[4] = -1;
+  nchild_fds = 5;
+  connect_flags = 0;
+#endif  /*!USE_DESCRIPTOR_PASSING*/
 
   pgmname = file_name ? file_name : _gpgme_get_default_gpgsm_name ();
 
@@ -339,6 +384,18 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
       argv[argc++] = "--homedir";
       argv[argc++] = home_dir;
     }
+  /* Set up diagnostics */
+  err = gpgme_data_new (&gpgsm->diagnostics);
+  if (err)
+    goto leave;
+  gpgsm->diag_cb.data = gpgsm->diagnostics;
+  argv[argc++] = "--logger-fd";
+  if (gpgrt_asprintf (&diag_fd_str, "%i", gpgsm->diag_cb.server_fd) == -1)
+    {
+      err = gpg_error_from_syserror ();
+      goto leave;
+    }
+  argv[argc++] = diag_fd_str;
   argv[argc++] = "--server";
   argv[argc++] = NULL;
 
@@ -349,26 +406,24 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
     goto leave;
   assuan_ctx_set_system_hooks (gpgsm->assuan_ctx, &_gpgme_assuan_system_hooks);
 
-#if USE_DESCRIPTOR_PASSING
-  err = assuan_pipe_connect (gpgsm->assuan_ctx, pgmname, argv,
-                             NULL, NULL, NULL, ASSUAN_PIPE_CONNECT_FDPASSING);
-#else
   {
-    assuan_fd_t achild_fds[4];
+    assuan_fd_t achild_fds[5];
     int i;
 
     /* For now... */
-    for (i = 0; i < 4; i++)
+    for (i = 0; i < nchild_fds; i++)
       achild_fds[i] = (assuan_fd_t) child_fds[i];
 
     err = assuan_pipe_connect (gpgsm->assuan_ctx, pgmname, argv,
-                               achild_fds, NULL, NULL, 0);
+                               achild_fds, NULL, NULL, connect_flags);
 
-    /* For now... */
-    for (i = 0; i < 4; i++)
+    /* FIXME: Check whether our Windows code still updates the list.*/
+    for (i = 0; i < nchild_fds; i++)
       child_fds[i] = (int) achild_fds[i];
   }
 
+
+#if !USE_DESCRIPTOR_PASSING
   /* On Windows, handles are inserted in the spawned process with
      DuplicateHandle, and child_fds contains the server-local names
      for the inserted handles when assuan_pipe_connect returns.  */
@@ -386,6 +441,8 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 		sizeof gpgsm->output_cb.server_fd_str, "%d", child_fds[1]);
       snprintf (gpgsm->message_cb.server_fd_str,
 		sizeof gpgsm->message_cb.server_fd_str, "%d", child_fds[2]);
+      snprintf (gpgsm->diag_cb.server_fd_str,
+		sizeof gpgsm->diag_cb.server_fd_str, "%d", child_fds[3]);
     }
 #endif
   if (err)
@@ -396,7 +453,7 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
     goto leave;
   if (dft_display)
     {
-      if (asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
+      if (gpgrt_asprintf (&optstr, "OPTION display=%s", dft_display) < 0)
         {
 	  free (dft_display);
 	  err = gpg_error_from_syserror ();
@@ -406,7 +463,7 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 
       err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL, NULL,
 			     NULL, NULL, NULL);
-      free (optstr);
+      gpgrt_free (optstr);
       if (err)
 	goto leave;
     }
@@ -430,14 +487,14 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 	 ways, e.g., when /dev/pts is not accessible under chroot.  */
       if (!rc)
 	{
-	  if (asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
+	  if (gpgrt_asprintf (&optstr, "OPTION ttyname=%s", dft_ttyname) < 0)
 	    {
 	      err = gpg_error_from_syserror ();
 	      goto leave;
 	    }
 	  err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL, NULL,
 				 NULL, NULL, NULL);
-	  free (optstr);
+	  gpgrt_free (optstr);
 	  if (err)
 	    goto leave;
 
@@ -446,7 +503,7 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 	    goto leave;
 	  if (dft_ttytype)
 	    {
-	      if (asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype) < 0)
+	      if (gpgrt_asprintf (&optstr, "OPTION ttytype=%s", dft_ttytype)< 0)
 		{
 		  free (dft_ttytype);
 		  err = gpg_error_from_syserror ();
@@ -456,7 +513,7 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 
 	      err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
 				     NULL, NULL, NULL, NULL);
-	      free (optstr);
+	      gpgrt_free (optstr);
 	      if (err)
 		goto leave;
 	    }
@@ -498,6 +555,12 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
       goto leave;
     }
 #endif
+  if (!err && _gpgme_io_set_close_notify (gpgsm->diag_cb.fd,
+                                          close_notify_handler, gpgsm))
+    {
+      err = gpg_error (GPG_ERR_GENERAL);
+      goto leave;
+    }
 
  leave:
   /* Close the server ends of the pipes (because of this, we must use
@@ -510,17 +573,37 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
     _gpgme_io_close (gpgsm->output_cb.server_fd);
   if (gpgsm->message_cb.server_fd != -1)
     _gpgme_io_close (gpgsm->message_cb.server_fd);
+  if (gpgsm->diag_cb.server_fd != -1)
+    _gpgme_io_close (gpgsm->diag_cb.server_fd);
 #endif
 
   if (err)
     gpgsm_release (gpgsm);
   else
     *engine = gpgsm;
-
+  free (diag_fd_str);
   return err;
 }
 
 
+/* Copy flags from CTX into the engine object.  */
+static void
+gpgsm_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
+{
+  engine_gpgsm_t gpgsm = engine;
+
+  if (ctx->request_origin)
+    {
+      if (strlen (ctx->request_origin) + 1 > sizeof gpgsm->request_origin)
+        strcpy (gpgsm->request_origin, "xxx"); /* Too long  - force error */
+      else
+        strcpy (gpgsm->request_origin, ctx->request_origin);
+    }
+  else
+    *gpgsm->request_origin = 0;
+}
+
+
 static gpgme_error_t
 gpgsm_set_locale (void *engine, int category, const char *value)
 {
@@ -561,13 +644,13 @@ gpgsm_set_locale (void *engine, int category, const char *value)
   if (!value)
     return 0;
 
-  if (asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
+  if (gpgrt_asprintf (&optstr, "OPTION %s=%s", catstr, value) < 0)
     err = gpg_error_from_syserror ();
   else
     {
       err = assuan_transact (gpgsm->assuan_ctx, optstr, NULL, NULL,
 			     NULL, NULL, NULL, NULL);
-      free (optstr);
+      gpgrt_free (optstr);
     }
 
   return err;
@@ -593,7 +676,7 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
     {
       err = assuan_read_line (ctx, &line, &linelen);
       if (err)
-	return err;
+	break;
 
       if (*line == '#' || !linelen)
 	continue;
@@ -601,7 +684,7 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
       if (linelen >= 2
 	  && line[0] == 'O' && line[1] == 'K'
 	  && (line[2] == '\0' || line[2] == ' '))
-	return cb_err;
+	break;
       else if (linelen >= 4
 	  && line[0] == 'E' && line[1] == 'R' && line[2] == 'R'
 	  && line[3] == ' ')
@@ -610,6 +693,7 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
              more related to gpgme and thus probably more important
              than the error returned by the engine.  */
           err = cb_err? cb_err : atoi (&line[4]);
+          cb_err = 0;
         }
       else if (linelen >= 2
 	       && line[0] == 'S' && line[1] == ' ')
@@ -646,10 +730,16 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
              to stop.  As with ERR we prefer a status callback
              generated error code, though.  */
           err = cb_err ? cb_err : gpg_error (GPG_ERR_GENERAL);
+          cb_err = 0;
         }
     }
   while (!err);
 
+  /* We only want the first error from the status handler, thus we
+   * take the one saved in CB_ERR. */
+  if (!err && cb_err)
+    err = cb_err;
+
   return err;
 }
 
@@ -810,7 +900,7 @@ status_handler (void *opaque, int fd)
 	{
 	  /* Try our best to terminate the connection friendly.  */
 	  /*	  assuan_write_line (gpgsm->assuan_ctx, "BYE"); */
-          TRACE3 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: error from assuan (%d) getting status line : %s",
                   fd, err, gpg_strerror (err));
 	}
@@ -822,7 +912,7 @@ status_handler (void *opaque, int fd)
 	    err = atoi (&line[4]);
 	  if (! err)
 	    err = gpg_error (GPG_ERR_GENERAL);
-          TRACE2 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: ERR line - mapped to: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 	  /* Try our best to terminate the connection friendly.  */
@@ -851,7 +941,7 @@ status_handler (void *opaque, int fd)
               gpgsm->colon.any = 0;
               err = gpgsm->colon.fnc (gpgsm->colon.fnc_value, NULL);
             }
-          TRACE2 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: OK line - final status: %s",
                   fd, err ? gpg_strerror (err) : "ok");
 	  _gpgme_io_close (gpgsm->status_cb.fd);
@@ -926,7 +1016,7 @@ status_handler (void *opaque, int fd)
 		    dst++;
 		}
 	    }
-          TRACE2 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: D line; final status: %s",
                   fd, err? gpg_strerror (err):"ok");
         }
@@ -959,8 +1049,7 @@ status_handler (void *opaque, int fd)
           while (linelen > 0)
             {
               nwritten = gpgme_data_write (gpgsm->inline_data, src, linelen);
-              if (!nwritten || (nwritten < 0 && errno != EINTR)
-                  || nwritten > linelen)
+              if (nwritten <= 0 || nwritten > linelen)
                 {
                   err = gpg_error_from_syserror ();
                   break;
@@ -969,7 +1058,7 @@ status_handler (void *opaque, int fd)
               linelen -= nwritten;
             }
 
-          TRACE2 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: D inlinedata; final status: %s",
                   fd, err? gpg_strerror (err):"ok");
         }
@@ -986,8 +1075,17 @@ status_handler (void *opaque, int fd)
 	    *(rest++) = 0;
 
 	  r = _gpgme_parse_status (line + 2);
+          if (gpgsm->status.mon_cb && r != GPGME_STATUS_PROGRESS)
+            {
+              /* Note that we call the monitor even if we do
+               * not know the status code (r < 0).  */
+              err = gpgsm->status.mon_cb (gpgsm->status.mon_cb_value,
+                                          line + 2, rest);
+            }
+          else
+            err = 0;
 
-	  if (r >= 0)
+	  if (r >= 0 && !err)
 	    {
 	      if (gpgsm->status.fnc)
                 {
@@ -998,7 +1096,7 @@ status_handler (void *opaque, int fd)
 	    }
 	  else
 	    fprintf (stderr, "[UNKNOWN STATUS]%s %s", line + 2, rest);
-          TRACE3 (DEBUG_CTX, "gpgme:status_handler", gpgsm,
+          TRACE (DEBUG_CTX, "gpgme:status_handler", gpgsm,
 		  "fd 0x%x: S line (%s) - final status: %s",
                   fd, line+2, err? gpg_strerror (err):"ok");
 	}
@@ -1028,8 +1126,8 @@ add_io_cb (engine_gpgsm_t gpgsm, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 {
   gpgme_error_t err;
 
-  TRACE_BEG2 (DEBUG_ENGINE, "engine-gpgsm:add_io_cb", gpgsm,
-              "fd %d, dir %d", iocbd->fd, iocbd->dir);
+  TRACE_BEG  (DEBUG_ENGINE, "engine-gpgsm:add_io_cb", gpgsm,
+              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
   err = (*gpgsm->io_cbs.add) (gpgsm->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -1051,6 +1149,20 @@ start (engine_gpgsm_t gpgsm, const char *command)
   int nfds;
   int i;
 
+  if (*gpgsm->request_origin)
+    {
+      char *cmd;
+
+      cmd = _gpgme_strconcat ("OPTION request-origin=",
+                              gpgsm->request_origin, NULL);
+      if (!cmd)
+        return gpg_error_from_syserror ();
+      err = gpgsm_assuan_simple_command (gpgsm, cmd, NULL, NULL);
+      free (cmd);
+      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
+        return err;
+    }
+
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
@@ -1091,6 +1203,8 @@ start (engine_gpgsm_t gpgsm, const char *command)
     err = add_io_cb (gpgsm, &gpgsm->output_cb, _gpgme_data_inbound_handler);
   if (!err && gpgsm->message_cb.fd != -1)
     err = add_io_cb (gpgsm, &gpgsm->message_cb, _gpgme_data_outbound_handler);
+  if (!err && gpgsm->diag_cb.fd != -1)
+    err = add_io_cb (gpgsm, &gpgsm->diag_cb, _gpgme_data_inbound_handler);
 
   if (!err)
     err = assuan_write_line (gpgsm->assuan_ctx, command);
@@ -1120,17 +1234,25 @@ gpgsm_reset (void *engine)
 
 
 static gpgme_error_t
-gpgsm_decrypt (void *engine, gpgme_data_t ciph, gpgme_data_t plain,
-               int export_session_key, const char *override_session_key)
+gpgsm_decrypt (void *engine,
+               gpgme_decrypt_flags_t flags,
+               gpgme_data_t ciph, gpgme_data_t plain,
+               int export_session_key, const char *override_session_key,
+               int auto_key_retrieve)
 {
   engine_gpgsm_t gpgsm = engine;
   gpgme_error_t err;
 
+  (void)flags;
+
   /* gpgsm is not capable of exporting session keys right now, so we
    * will ignore this if requested. */
   (void)export_session_key;
   (void)override_session_key;
 
+  /* --auto-key-retrieve is also not supported.  */
+  (void)auto_key_retrieve;
+
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
@@ -1151,7 +1273,7 @@ gpgsm_decrypt (void *engine, gpgme_data_t ciph, gpgme_data_t plain,
 
 
 static gpgme_error_t
-gpgsm_delete (void *engine, gpgme_key_t key, int allow_secret)
+gpgsm_delete (void *engine, gpgme_key_t key, unsigned int flags)
 {
   engine_gpgsm_t gpgsm = engine;
   gpgme_error_t err;
@@ -1160,7 +1282,7 @@ gpgsm_delete (void *engine, gpgme_key_t key, int allow_secret)
   char *line;
   int length = 8;	/* "DELKEYS " */
 
-  (void)allow_secret;
+  (void)flags;
 
   if (!fpr)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -1278,8 +1400,65 @@ set_recipients (engine_gpgsm_t gpgsm, gpgme_key_t recp[])
 }
 
 
+/* Take recipients from the LF delimited STRING and send RECIPIENT
+ * commands to gpgsm.  */
 static gpgme_error_t
-gpgsm_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
+set_recipients_from_string (engine_gpgsm_t gpgsm, const char *string)
+{
+  gpg_error_t err = 0;
+  char *line = NULL;
+  int ignore = 0;
+  int any = 0;
+  const char *s;
+  int n;
+
+  do
+    {
+      while (*string == ' ' || *string == '\t')
+        string++;
+      if (!*string)
+        break;
+
+      s = strchr (string, '\n');
+      if (s)
+        n = s - string;
+      else
+        n = strlen (string);
+      while (n && (string[n-1] == ' ' || string[n-1] == '\t'))
+        n--;
+
+      if (!ignore && n == 2 && !memcmp (string, "--", 2))
+        ignore = 1;
+      else if (!ignore && n > 2 && !memcmp (string, "--", 2))
+        err = gpg_error (GPG_ERR_UNKNOWN_OPTION);
+      else if (n) /* Not empty - use it.  */
+        {
+          gpgrt_free (line);
+          if (gpgrt_asprintf (&line, "RECIPIENT %.*s", n, string) < 0)
+            err = gpg_error_from_syserror ();
+          else
+            {
+              err = gpgsm_assuan_simple_command (gpgsm, line, gpgsm->status.fnc,
+                                                 gpgsm->status.fnc_value);
+              if (!err)
+                any = 1;
+            }
+        }
+
+      string += n + !!s;
+    }
+  while (!err);
+
+  if (!err && !any)
+    err = gpg_error (GPG_ERR_MISSING_KEY);
+  gpgrt_free (line);
+  return err;
+}
+
+
+static gpgme_error_t
+gpgsm_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
+               gpgme_encrypt_flags_t flags,
 	       gpgme_data_t plain, gpgme_data_t ciph, int use_armor)
 {
   engine_gpgsm_t gpgsm = engine;
@@ -1287,10 +1466,10 @@ gpgsm_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
 
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
-  if (!recp)
+  if (!recp && !recpstring) /* Symmetric only */
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
-  if (flags & GPGME_ENCRYPT_NO_ENCRYPT_TO)
+  if ((flags & GPGME_ENCRYPT_NO_ENCRYPT_TO))
     {
       err = gpgsm_assuan_simple_command (gpgsm,
 					 "OPTION no-encrypt-to", NULL, NULL);
@@ -1310,7 +1489,10 @@ gpgsm_encrypt (void *engine, gpgme_key_t recp[], gpgme_encrypt_flags_t flags,
   gpgsm_clear_fd (gpgsm, MESSAGE_FD);
   gpgsm->inline_data = NULL;
 
-  err = set_recipients (gpgsm, recp);
+  if (!recp && recpstring)
+    err = set_recipients_from_string (gpgsm, recpstring);
+  else
+    err = set_recipients (gpgsm, recp);
 
   if (!err)
     err = start (gpgsm, "ENCRYPT");
@@ -1553,7 +1735,7 @@ gpgsm_import (void *engine, gpgme_data_t keydata, gpgme_key_t *keyarray)
               && *keyarray[idx]->subkeys->fpr)
             buflen += strlen (keyarray[idx]->subkeys->fpr) + 1;
         }
-      /* Allocate a bufer with extra space for the trailing Nul
+      /* Allocate a buffer with extra space for the trailing Nul
          introduced by the use of stpcpy.  */
       buffer = malloc (buflen+1);
       if (!buffer)
@@ -1642,10 +1824,10 @@ gpgsm_keylist (void *engine, const char *pattern, int secret_only,
     gpgsm_assuan_simple_command (gpgsm, "GETINFO agent-check", NULL, NULL);
 
   /* Always send list-mode option because RESET does not reset it.  */
-  if (asprintf (&line, "OPTION list-mode=%d", (list_mode & 3)) < 0)
+  if (gpgrt_asprintf (&line, "OPTION list-mode=%d", (list_mode & 3)) < 0)
     return gpg_error_from_syserror ();
   err = gpgsm_assuan_simple_command (gpgsm, line, NULL, NULL);
-  free (line);
+  gpgrt_free (line);
   if (err)
     return err;
 
@@ -1726,10 +1908,10 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
     list_mode |= 2;
 
   /* Always send list-mode option because RESET does not reset it.  */
-  if (asprintf (&line, "OPTION list-mode=%d", (list_mode & 3)) < 0)
+  if (gpgrt_asprintf (&line, "OPTION list-mode=%d", (list_mode & 3)) < 0)
     return gpg_error_from_syserror ();
   err = gpgsm_assuan_simple_command (gpgsm, line, NULL, NULL);
-  free (line);
+  gpgrt_free (line);
   if (err)
     return err;
 
@@ -1860,10 +2042,11 @@ gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
 	 can reset any previously set value in case the default is
 	 requested.  */
 
-      if (asprintf (&assuan_cmd, "OPTION include-certs %i", include_certs) < 0)
+      if (gpgrt_asprintf (&assuan_cmd,
+                          "OPTION include-certs %i", include_certs) < 0)
 	return gpg_error_from_syserror ();
       err = gpgsm_assuan_simple_command (gpgsm, assuan_cmd, NULL, NULL);
-      free (assuan_cmd);
+      gpgrt_free (assuan_cmd);
       if (err)
 	return err;
     }
@@ -1921,11 +2104,19 @@ gpgsm_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
   if (err)
     return err;
-  if (plaintext)
+  if (!signed_text)
     {
       /* Normal or cleartext signature.  */
-      gpgsm->output_cb.data = plaintext;
-      err = gpgsm_set_fd (gpgsm, OUTPUT_FD, 0);
+      if (plaintext)
+        {
+          gpgsm->output_cb.data = plaintext;
+          err = gpgsm_set_fd (gpgsm, OUTPUT_FD, 0);
+        }
+      else
+        {
+          /* No output requested.  */
+          gpgsm_clear_fd (gpgsm, OUTPUT_FD);
+        }
       gpgsm_clear_fd (gpgsm, MESSAGE_FD);
     }
   else
@@ -1952,9 +2143,40 @@ gpgsm_getauditlog (void *engine, gpgme_data_t output, unsigned int flags)
   engine_gpgsm_t gpgsm = engine;
   gpgme_error_t err = 0;
 
+
   if (!gpgsm || !output)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if ((flags & GPGME_AUDITLOG_DIAG) && (flags & GPGME_AUDITLOG_HTML))
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  if ((flags & GPGME_AUDITLOG_DIAG))
+    {
+      char buf[BUFFER_SIZE];
+      int nread;
+      int any_written = 0;
+      gpgme_data_rewind (gpgsm->diagnostics);
+
+      while ((nread = gpgme_data_read (gpgsm->diagnostics,
+                                       buf, BUFFER_SIZE)) > 0)
+        {
+          any_written = 1;
+          if (gpgme_data_write (output, buf, nread) == -1)
+            return gpg_error_from_syserror ();
+        }
+      if (!any_written)
+        return gpg_error (GPG_ERR_NO_DATA);
+
+      if (nread == -1)
+        return gpg_error_from_syserror ();
+
+      gpgme_data_rewind (output);
+      return 0;
+    }
+
+  if (!gpgsm->assuan_ctx)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
 #if USE_DESCRIPTOR_PASSING
   gpgsm->output_cb.data = output;
   err = gpgsm_set_fd (gpgsm, OUTPUT_FD, 0);
@@ -2028,7 +2250,7 @@ gpgsm_io_event (void *engine, gpgme_event_io_t type, void *type_data)
 {
   engine_gpgsm_t gpgsm = engine;
 
-  TRACE3 (DEBUG_ENGINE, "gpgme:gpgsm_io_event", gpgsm,
+  TRACE (DEBUG_ENGINE, "gpgme:gpgsm_io_event", gpgsm,
           "event %p, type %d, type_data %p",
           gpgsm->io_cbs.event, type, type_data);
   if (gpgsm->io_cbs.event)
@@ -2048,7 +2270,7 @@ gpgsm_passwd (void *engine, gpgme_key_t key, unsigned int flags)
   if (!key || !key->subkeys || !key->subkeys->fpr)
     return gpg_error (GPG_ERR_INV_CERT_OBJ);
 
-  if (asprintf (&line, "PASSWD -- %s", key->subkeys->fpr) < 0)
+  if (gpgrt_asprintf (&line, "PASSWD -- %s", key->subkeys->fpr) < 0)
     return gpg_error_from_syserror ();
 
   gpgsm_clear_fd (gpgsm, OUTPUT_FD);
@@ -2057,7 +2279,7 @@ gpgsm_passwd (void *engine, gpgme_key_t key, unsigned int flags)
   gpgsm->inline_data = NULL;
 
   err = start (gpgsm, line);
-  free (line);
+  gpgrt_free (line);
 
   return err;
 }
@@ -2086,7 +2308,7 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     gpgsm_set_colon_line_handler,
     gpgsm_set_locale,
     NULL,		/* set_protocol */
-    gpgsm_decrypt,
+    gpgsm_set_engine_flags,
     gpgsm_decrypt,
     gpgsm_delete,	/* decrypt_verify */
     NULL,		/* edit */
@@ -2098,6 +2320,7 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     gpgsm_import,
     gpgsm_keylist,
     gpgsm_keylist_ext,
+    NULL,               /* keylist_data */
     NULL,               /* keysign */
     NULL,               /* tofu_policy */
     gpgsm_sign,
@@ -2107,6 +2330,7 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     NULL,               /* opassuan_transact */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
+    NULL,		/* conf_dir */
     NULL,               /* query_swdb */
     gpgsm_set_io_cbs,
     gpgsm_io_event,
