@@ -1,22 +1,23 @@
 /* keylist.c - Listing keys.
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007,
-                 2008, 2009  g10 Code GmbH
-
-   This file is part of GPGME.
-
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, see <https://www.gnu.org/licenses/>.
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2006, 2007,
+ *               2008, 2009  g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #if HAVE_CONFIG_H
@@ -100,19 +101,19 @@ gpgme_op_keylist_result (gpgme_ctx_t ctx)
   op_data_t opd;
   gpgme_error_t err;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_result", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_result", ctx, "");
 
   err = _gpgme_op_data_lookup (ctx, OPDATA_KEYLIST, &hook, -1, NULL);
   opd = hook;
   if (err || !opd)
     {
-      TRACE_SUC0 ("result=(null)");
+      TRACE_SUC ("result=(null)");
       return NULL;
     }
 
-  TRACE_LOG1 ("truncated = %i", opd->result.truncated);
+  TRACE_LOG  ("truncated = %i", opd->result.truncated);
 
-  TRACE_SUC1 ("result=%p", &opd->result);
+  TRACE_SUC ("result=%p", &opd->result);
   return &opd->result;
 }
 
@@ -571,7 +572,7 @@ keylist_colon_handler (void *priv, char *line)
 
   key = opd->tmp_key;
 
-  TRACE2 (DEBUG_CTX, "gpgme:keylist_colon_handler", ctx,
+  TRACE (DEBUG_CTX, "gpgme:keylist_colon_handler", ctx,
 	  "key = %p, line = %s", key, line ? line : "(null)");
 
   if (!line)
@@ -1077,7 +1078,7 @@ gpgme_op_keylist_start (gpgme_ctx_t ctx, const char *pattern, int secret_only)
   op_data_t opd;
   int flags = 0;
 
-  TRACE_BEG2 (DEBUG_CTX, "gpgme_op_keylist_start", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_op_keylist_start", ctx,
 	      "pattern=%s, secret_only=%i", pattern, secret_only);
 
   if (!ctx)
@@ -1121,7 +1122,7 @@ gpgme_op_keylist_ext_start (gpgme_ctx_t ctx, const char *pattern[],
   op_data_t opd;
   int flags = 0;
 
-  TRACE_BEG2 (DEBUG_CTX, "gpgme_op_keylist_ext_start", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_op_keylist_ext_start", ctx,
 	      "secret_only=%i, reserved=0x%x", secret_only, reserved);
 
   if (!ctx)
@@ -1163,7 +1164,7 @@ gpgme_op_keylist_from_data_start (gpgme_ctx_t ctx, gpgme_data_t data,
   void *hook;
   op_data_t opd;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_from_data_start", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_from_data_start", ctx, "");
 
   if (!ctx || !data || reserved)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -1198,7 +1199,7 @@ gpgme_op_keylist_next (gpgme_ctx_t ctx, gpgme_key_t *r_key)
   void *hook;
   op_data_t opd;
 
-  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_next", ctx);
+  TRACE_BEG (DEBUG_CTX, "gpgme_op_keylist_next", ctx, "");
 
   if (!ctx || !r_key)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
@@ -1234,9 +1235,10 @@ gpgme_op_keylist_next (gpgme_ctx_t ctx, gpgme_key_t *r_key)
   *r_key = queue_item->key;
   free (queue_item);
 
-  return TRACE_SUC2 ("key=%p (%s)", *r_key,
-		     ((*r_key)->subkeys && (*r_key)->subkeys->fpr) ?
-		     (*r_key)->subkeys->fpr : "invalid");
+  TRACE_SUC ("key=%p (%s)", *r_key,
+             ((*r_key)->subkeys && (*r_key)->subkeys->fpr) ?
+             (*r_key)->subkeys->fpr : "invalid");
+  return 0;
 }
 
 
@@ -1244,7 +1246,7 @@ gpgme_op_keylist_next (gpgme_ctx_t ctx, gpgme_key_t *r_key)
 gpgme_error_t
 gpgme_op_keylist_end (gpgme_ctx_t ctx)
 {
-  TRACE (DEBUG_CTX, "gpgme_op_keylist_end", ctx);
+  TRACE (DEBUG_CTX, "gpgme_op_keylist_end", ctx, "");
 
   if (!ctx)
     return gpg_error (GPG_ERR_INV_VALUE);
@@ -1261,11 +1263,14 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
 {
   gpgme_ctx_t listctx;
   gpgme_error_t err;
-  gpgme_key_t key;
+  gpgme_key_t result, key;
 
-  TRACE_BEG2 (DEBUG_CTX, "gpgme_get_key", ctx,
+  TRACE_BEG  (DEBUG_CTX, "gpgme_get_key", ctx,
 	      "fpr=%s, secret=%i", fpr, secret);
 
+  if (r_key)
+    *r_key = NULL;
+
   if (!ctx || !r_key || !fpr)
     return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
 
@@ -1295,7 +1300,7 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
 
   err = gpgme_op_keylist_start (listctx, fpr, secret);
   if (!err)
-    err = gpgme_op_keylist_next (listctx, r_key);
+    err = gpgme_op_keylist_next (listctx, &result);
   if (!err)
     {
     try_next_key:
@@ -1305,9 +1310,9 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
       else
 	{
           if (!err
-              && *r_key && (*r_key)->subkeys && (*r_key)->subkeys->fpr
+              && result && result->subkeys && result->subkeys->fpr
               && key && key->subkeys && key->subkeys->fpr
-              && !strcmp ((*r_key)->subkeys->fpr, key->subkeys->fpr))
+              && !strcmp (result->subkeys->fpr, key->subkeys->fpr))
             {
               /* The fingerprint is identical.  We assume that this is
                  the same key and don't mark it as an ambiguous.  This
@@ -1323,13 +1328,15 @@ gpgme_get_key (gpgme_ctx_t ctx, const char *fpr, gpgme_key_t *r_key,
 	      gpgme_key_unref (key);
 	      err = gpg_error (GPG_ERR_AMBIGUOUS_NAME);
 	    }
-	  gpgme_key_unref (*r_key);
+	  gpgme_key_unref (result);
+          result = NULL;
 	}
     }
   gpgme_release (listctx);
   if (! err)
     {
-      TRACE_LOG2 ("key=%p (%s)", *r_key,
+      *r_key = result;
+      TRACE_LOG  ("key=%p (%s)", *r_key,
 		  ((*r_key)->subkeys && (*r_key)->subkeys->fpr) ?
 		  (*r_key)->subkeys->fpr : "invalid");
     }
