@@ -16,8 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://www.gnu.org/licenses/>.
- **/
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -65,13 +66,22 @@
 #include "sys-util.h"
 
 
-#ifndef HAVE_W32CE_SYSTEM
 #define HAVE_ALLOW_SET_FOREGROUND_WINDOW 1
-#endif
 #ifndef F_OK
 # define F_OK 0
 #endif
 
+/* The Registry key used by GNUPG.  */
+#ifdef _WIN64
+# define GNUPG_REGKEY_2  "Software\\Wow6432Node\\GNU\\GnuPG"
+#else
+# define GNUPG_REGKEY_2  "Software\\GNU\\GnuPG"
+#endif
+#ifdef _WIN64
+# define GNUPG_REGKEY_3  "Software\\Wow6432Node\\GnuPG"
+#else
+# define GNUPG_REGKEY_3  "Software\\GnuPG"
+#endif
 
 DEFINE_STATIC_LOCK (get_path_lock);
 
@@ -90,8 +100,6 @@ static char *default_gpgconf_name;
    gpgme_set_global_flag and accessed by _gpgme_get_inst_dir.  */
 static char *override_inst_dir;
 
-#ifdef HAVE_ALLOW_SET_FOREGROUND_WINDOW
-
 #define RTLD_LAZY 0
 
 static GPG_ERR_INLINE void *
@@ -126,7 +134,6 @@ dlclose (void * hd)
     }
   return -1;
 }
-#endif /* HAVE_ALLOW_SET_FOREGROUND_WINDOW */
 
 
 /* Return a malloced string encoded in UTF-8 from the wide char input
@@ -161,6 +168,48 @@ wchar_to_utf8 (const wchar_t *string)
 }
 
 
+/* Return a malloced wide char string from an UTF-8 encoded input
+   string STRING.  Caller must free this value. On failure returns
+   NULL; caller may use GetLastError to get the actual error number.
+   Calling this function with STRING set to NULL is not defined. */
+static wchar_t *
+utf8_to_wchar (const char *string)
+{
+  int n;
+  wchar_t *result;
+
+
+  n = MultiByteToWideChar (CP_UTF8, 0, string, -1, NULL, 0);
+  if (n < 0)
+    return NULL;
+
+  result = (wchar_t *) malloc ((n+1) * sizeof *result);
+  if (!result)
+    return NULL;
+
+  n = MultiByteToWideChar (CP_UTF8, 0, string, -1, result, n);
+  if (n < 0)
+    {
+      free (result);
+      return NULL;
+    }
+  return result;
+}
+
+
+/* Same as utf8_to_wchar but calling it with NULL returns
+   NULL.  So a return value of NULL only indicates failure
+   if STRING is not set to NULL. */
+static wchar_t *
+utf8_to_wchar0 (const char *string)
+{
+  if (!string)
+    return NULL;
+
+  return utf8_to_wchar (string);
+}
+
+
 /* Replace all forward slashes by backslashes.  */
 static void
 replace_slashes (char *string)
@@ -213,25 +262,66 @@ _gpgme_allow_set_foreground_window (pid_t pid)
 
   if (!pid || pid == (pid_t)(-1))
     {
-      TRACE1 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+      TRACE (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", NULL,
 	      "no action for pid %d", (int)pid);
     }
   else if (func)
     {
       int rc = func (pid);
-      TRACE2 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+      TRACE (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", NULL,
 	      "called for pid %d; result=%d", (int)pid, rc);
 
     }
   else
     {
-      TRACE0 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+      TRACE (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", NULL,
 	      "function not available");
     }
 #endif /* HAVE_ALLOW_SET_FOREGROUND_WINDOW */
 }
 
 
+/* Wrapper around CancelSynchronousIo which is only available since
+ * Vista.  */
+void
+_gpgme_w32_cancel_synchronous_io (HANDLE thread)
+{
+  static int initialized;
+  static BOOL (WINAPI * func)(DWORD);
+  void *handle;
+
+  if (!initialized)
+    {
+      /* Available since Vista; thus we dynload it.  */
+      initialized = 1;
+      handle = dlopen ("kernel32.dll", RTLD_LAZY);
+      if (handle)
+        {
+          func = dlsym (handle, "CancelSynchronousIo");
+          if (!func)
+            {
+              dlclose (handle);
+              handle = NULL;
+            }
+        }
+    }
+
+  if (func)
+    {
+      if (!func (thread) && GetLastError() != ERROR_NOT_FOUND)
+        {
+          TRACE (DEBUG_ENGINE, "gpgme:CancelSynchronousIo", NULL,
+                  "called for thread %p: ec=%d", thread, GetLastError ());
+        }
+    }
+  else
+    {
+      TRACE (DEBUG_ENGINE, "gpgme:CancelSynchronousIo", NULL,
+	      "function not available");
+    }
+}
+
+
 /* Return a string from the W32 Registry or NULL in case of error.
    Caller must release the return value.  A NULL for root is an alias
    for HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE in turn. */
@@ -292,53 +382,6 @@ read_w32_registry_string (const char *root, const char *dir, const char *name)
     }
   result[nbytes] = 0; /* Make sure it is really a string.  */
 
-#ifndef HAVE_W32CE_SYSTEM
-  /* Windows CE does not have an environment.  */
-  if (type == REG_EXPAND_SZ && strchr (result, '%'))
-    {
-      char *tmp;
-
-      n1 += 1000;
-      tmp = malloc (n1 + 1);
-      if (!tmp)
-        goto leave;
-      nbytes = ExpandEnvironmentStrings (result, tmp, n1);
-      if (nbytes && nbytes > n1)
-        {
-          free (tmp);
-          n1 = nbytes;
-          tmp = malloc (n1 + 1);
-          if (!tmp)
-            goto leave;
-          nbytes = ExpandEnvironmentStrings (result, tmp, n1);
-          if (nbytes && nbytes > n1) {
-            free (tmp); /* Oops - truncated, better don't expand at all. */
-            goto leave;
-          }
-          tmp[nbytes] = 0;
-          free (result);
-          result = tmp;
-        }
-      else if (nbytes)  /* Okay, reduce the length. */
-        {
-          tmp[nbytes] = 0;
-          free (result);
-          result = malloc (strlen (tmp)+1);
-          if (!result)
-            result = tmp;
-          else
-            {
-              strcpy (result, tmp);
-              free (tmp);
-            }
-        }
-      else  /* Error - don't expand. */
-        {
-          free (tmp);
-        }
-    }
-#endif
-
  leave:
   RegCloseKey (key_handle);
   return result;
@@ -394,7 +437,7 @@ find_program_in_dir (const char *dir, const char *name)
   if (!result)
     return NULL;
 
-  if (access (result, F_OK))
+  if (_gpgme_access (result, F_OK))
     {
       free (result);
       return NULL;
@@ -407,28 +450,32 @@ find_program_in_dir (const char *dir, const char *name)
 static char *
 find_program_at_standard_place (const char *name)
 {
-  char path[MAX_PATH];
+  wchar_t path[MAX_PATH];
   char *result = NULL;
 
-  /* See http://wiki.tcl.tk/17492 for details on compatibility.
+  /* See https://wiki.tcl-lang.org/page/Getting+Windows+%22special+folders%22+with+Ffidl for details on compatibility.
 
      We First try the generic place and then fallback to the x86
      (i.e. 32 bit) place.  This will prefer a 64 bit of the program
      over a 32 bit version on 64 bit Windows if installed.  */
-  if (SHGetSpecialFolderPathA (NULL, path, CSIDL_PROGRAM_FILES, 0))
+  if (SHGetSpecialFolderPathW (NULL, path, CSIDL_PROGRAM_FILES, 0))
     {
-      result = _gpgme_strconcat (path, "\\", name, NULL);
-      if (result && access (result, F_OK))
+      char *utf8_path = wchar_to_utf8 (path);
+      result = _gpgme_strconcat (utf8_path, "\\", name, NULL);
+      free (utf8_path);
+      if (result && _gpgme_access (result, F_OK))
         {
           free (result);
           result = NULL;
         }
     }
   if (!result
-      && SHGetSpecialFolderPathA (NULL, path, CSIDL_PROGRAM_FILESX86, 0))
+      && SHGetSpecialFolderPathW (NULL, path, CSIDL_PROGRAM_FILESX86, 0))
     {
-      result = _gpgme_strconcat (path, "\\", name, NULL);
-      if (result && access (result, F_OK))
+      char *utf8_path = wchar_to_utf8 (path);
+      result = _gpgme_strconcat (utf8_path, "\\", name, NULL);
+      free (utf8_path);
+      if (result && _gpgme_access (result, F_OK))
         {
           free (result);
           result = NULL;
@@ -513,7 +560,7 @@ _gpgme_get_gpg_path (void)
       char *dir;
 
       dir = read_w32_registry_string ("HKEY_LOCAL_MACHINE",
-                                      "Software\\GNU\\GnuPG",
+                                      GNUPG_REGKEY_2,
                                       "Install Directory");
       if (dir)
         {
@@ -531,7 +578,8 @@ _gpgme_get_gpg_path (void)
 
   /* 4. Print a debug message if not found.  */
   if (!gpg)
-    _gpgme_debug (DEBUG_ENGINE, "_gpgme_get_gpg_path: '%s' not found", name);
+    _gpgme_debug (NULL, DEBUG_ENGINE, -1, NULL, NULL, NULL,
+                  "_gpgme_get_gpg_path: '%s' not found", name);
 
   return gpg;
 }
@@ -568,18 +616,19 @@ _gpgme_get_gpgconf_path (void)
       char *dir;
 
       dir = read_w32_registry_string (NULL,
-                                      "Software\\GNU\\GnuPG",
+                                      GNUPG_REGKEY_2,
                                       "Install Directory");
       if (!dir)
         {
           char *tmp = read_w32_registry_string (NULL,
-                                                "Software\\GnuPG",
+                                                GNUPG_REGKEY_3,
                                                 "Install Directory");
           if (tmp)
             {
-              if (gpgrt_asprintf (&dir, "%s\\bin", tmp) == -1)
-                return NULL;
+              dir = _gpgme_strconcat (tmp, "\\bin", NULL);
               free (tmp);
+              if (!dir)
+                return NULL;
             }
         }
       if (dir)
@@ -595,9 +644,18 @@ _gpgme_get_gpgconf_path (void)
       gpgconf = find_program_at_standard_place ("GNU\\GnuPG\\gpgconf.exe");
     }
 
+  /* 5. Try to find gpgconf.exe relative to us.  */
+  if (!gpgconf && inst_dir)
+    {
+      char *dir = _gpgme_strconcat (inst_dir, "\\..\\..\\GnuPG\\bin", NULL);
+      gpgconf = find_program_in_dir (dir, name);
+      free (dir);
+    }
+
   /* 5. Print a debug message if not found.  */
   if (!gpgconf)
-    _gpgme_debug (DEBUG_ENGINE, "_gpgme_get_gpgconf_path: '%s' not found",name);
+    _gpgme_debug (NULL, DEBUG_ENGINE, -1, NULL, NULL, NULL,
+                  "_gpgme_get_gpgconf_path: '%s' not found",name);
 
   return gpgconf;
 }
@@ -632,15 +690,8 @@ _gpgme_get_conf_int (const char *key, int *value)
   return 1;
 }
 
-
-#ifdef HAVE_W32CE_SYSTEM
-int
-_gpgme_mkstemp (int *fd, char **name)
-{
-  return -1;
-}
-#else
 
+
 /* mkstemp extracted from libc/sysdeps/posix/tempname.c.  Copyright
    (C) 1991-1999, 2000, 2001, 2006 Free Software Foundation, Inc.
 
@@ -774,30 +825,76 @@ _gpgme_mkstemp (int *fd, char **name)
   *name = tmpname;
   return 0;
 }
-#endif
 
 
-
-#ifdef HAVE_W32CE_SYSTEM
-/* Return a malloced string with the replacement value for the
-   GPGME_DEBUG envvar.  Caller must release.  Returns NULL if not
-   set.  */
-char *
-_gpgme_w32ce_get_debug_envvar (void)
+/* Like access but using windows _waccess */
+int
+_gpgme_access (const char *path, int mode)
 {
-  char *tmp;
+  wchar_t *u16 = utf8_to_wchar0 (path);
+  int r = _waccess (u16, mode);
 
-  tmp = read_w32_registry_string (NULL, "\\Software\\GNU\\gpgme", "debug");
-  if (tmp && !*tmp)
-    {
-      free (tmp);
-      tmp = NULL;
-    }
-  return tmp;
+  free(u16);
+  return r;
 }
-#endif /*HAVE_W32CE_SYSTEM*/
 
 
+/* Like CreateProcessA but mapping the arguments to wchar API */
+int _gpgme_create_process_utf8 (const char *application_name_utf8,
+                                char *command_line_utf8,
+                                LPSECURITY_ATTRIBUTES lpProcessAttributes,
+                                LPSECURITY_ATTRIBUTES lpThreadAttributes,
+                                BOOL bInheritHandles,
+                                DWORD dwCreationFlags,
+                                void *lpEnvironment,
+                                char *working_directory_utf8,
+                                LPSTARTUPINFOA si,
+                                LPPROCESS_INFORMATION lpProcessInformation)
+{
+  BOOL ret;
+  wchar_t *application_name = utf8_to_wchar0 (application_name_utf8);
+  wchar_t *command_line = utf8_to_wchar0 (command_line_utf8);
+  wchar_t *working_directory = utf8_to_wchar0 (working_directory_utf8);
+
+  STARTUPINFOW siw;
+  memset (&siw, 0, sizeof siw);
+  if (si)
+    {
+      siw.cb = sizeof (siw);
+      siw.dwFlags = si->dwFlags;
+      siw.wShowWindow = si->wShowWindow;
+      siw.hStdInput = si->hStdInput;
+      siw.hStdOutput = si->hStdOutput;
+      siw.hStdError = si->hStdError;
+      siw.dwX = si->dwX;
+      siw.dwY = si->dwY;
+      siw.dwXSize = si->dwXSize;
+      siw.dwYSize = si->dwYSize;
+      siw.dwXCountChars = si->dwXCountChars;
+      siw.dwYCountChars = si->dwYCountChars;
+      siw.dwFillAttribute = si->dwFillAttribute;
+      siw.lpDesktop = utf8_to_wchar0 (si->lpDesktop);
+      siw.lpTitle = utf8_to_wchar0 (si->lpTitle);
+    }
+
+  ret = CreateProcessW (application_name,
+                        command_line,
+                        lpProcessAttributes,
+                        lpThreadAttributes,
+                        bInheritHandles,
+                        dwCreationFlags,
+                        lpEnvironment,
+                        working_directory,
+                        si ? &siw : NULL,
+                        lpProcessInformation);
+  free (siw.lpTitle);
+  free (siw.lpDesktop);
+  free (application_name);
+  free (command_line);
+  free (working_directory);
+  return ret;
+}
+
 /* Entry point called by the DLL loader.  */
 #ifdef DLL_EXPORT
 int WINAPI
